from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getUtility, getMultiAdapter
from zope.container.interfaces import INameChooser

from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletAssignmentMapping


from plone.app.portlets.interfaces import IPortletPermissionChecker

from plone.app.portlets.portlets import calendar
from plone.app.portlets.portlets import classic
from plone.app.portlets.portlets import login
from plone.app.portlets.portlets import news
from plone.app.portlets.portlets import events
from plone.app.portlets.portlets import navigation
from plone.app.portlets.portlets import recent
from plone.app.portlets.portlets import review


from Acquisition import aq_base, aq_inner, aq_parent
from Products.CMFCore.utils import getToolByName

from Products.CMFPlone.interfaces import IPloneSiteRoot
from Acquisition.interfaces import IAcquirer


DONT_MIGRATE = object()

import transaction

class MigrateLegacyPortlets(BrowserView):
    """Migrate all legacy portlets, and move all portlets to the right column"""

    template = ViewPageTemplateFile('migrated_portlets_report.pt')
    migrated = []
    moved = []
    errors = []
    toCheck = []
    def convert_legacy_portlets_for_object(self, obj):
        portletsMapping = { 'portlet_login'      : login.Assignment(),
                            'portlet_news'       : news.Assignment(count=5),
                            'portlet_events'     : events.Assignment(count=5),
                            'portlet_navigation' : navigation.Assignment(),
                            'portlet_calendar'   : calendar.Assignment(),
                            'portlet_review'     : review.Assignment(),
                            'portlet_recent'     : recent.Assignment(count=5),
                            'portlet_related'    : DONT_MIGRATE,
                            'portlet_languages'  : DONT_MIGRATE,
                          }

        left = getUtility(IPortletManager, name='plone.leftcolumn')
        right = getUtility(IPortletManager, name='plone.rightcolumn')

        left_slots = getattr(aq_base(obj),'left_slots',[])
        right_slots = getattr(aq_base(obj),'right_slots',[])
            
        try:
            leftAssignable = getMultiAdapter((obj, left), IPortletAssignmentMapping).__of__(obj)
            rightAssignable = getMultiAdapter((obj, right), IPortletAssignmentMapping).__of__(obj)
            self.toCheck.append(obj)
        except:
            return
            
        IPortletPermissionChecker(leftAssignable)()
        IPortletPermissionChecker(rightAssignable)()
    
        leftChooser = INameChooser(leftAssignable)
        rightChooser = INameChooser(rightAssignable)

        #Move portlets from left column to right
        if len(leftAssignable.values()) > 0:
            movedObject = {'object':obj}
            movedObject['before'] = {'left':leftAssignable.items(),'right':rightAssignable.items()}
            for key,portlet in leftAssignable.items():
                rightAssignable[rightChooser.chooseName(None, portlet)] = portlet
                del(leftAssignable[key])
            movedObject['after'] = {'left':leftAssignable.items(),'right':rightAssignable.items()}
            self.moved.append(movedObject);
        if left_slots or right_slots:
            pass
        else:
            return

        #Convert Legacy Portlets from the left slot and move them to the right column
        currentObject = {'object':obj}
        currentObject['before'] = {'left_slot':[],'right_slot':[]}
        currentObject['after'] = {}
        is_iterable_left = True
        if left_slots:
            try:
                iter(left_slots)
            except:
                is_iterable_left = False
                self.errors.append({'object':obj,'left':left_slots,'right':''})
            if is_iterable_left:
                currentObject['before']['left_slot'] = left_slots
                for item in left_slots:
                    path = item.split('/')
                    if len(path) == 4:
                        newPortlet = portletsMapping.get(path[1], None)
                        if newPortlet is None and path[0] in ('context', 'here',) and path[2] == 'macros':
                            newPortlet = classic.Assignment(path[1], path[3])
                        if newPortlet is not None and newPortlet is not DONT_MIGRATE:
#                             leftAssignable[leftChooser.chooseName(None, newPortlet)] = newPortlet
                            rightAssignable[rightChooser.chooseName(None, newPortlet)] = newPortlet
                currentObject['after']['right'] = rightAssignable.items()
                                
        #Convert Legacy Portlets from the right slot
        is_iterable_right = True
        if right_slots:
            try:
                iter(right_slots)
            except:
                is_iterable_right = False
                self.errors.append({'object':obj,'right':right_slots,'left':''})
            if is_iterable_right:
                currentObject['before']['right_slot'] = right_slots
                for item in right_slots:
                    path = item.split('/')
                    if len(path) == 4:
                        newPortlet = portletsMapping.get(path[1], None)
                        if newPortlet is None and path[0] in ('context', 'here',) and path[2] == 'macros':
                            newPortlet = classic.Assignment(path[1], path[3])
                        if newPortlet is not None and newPortlet is not DONT_MIGRATE:
                            rightAssignable[rightChooser.chooseName(None, newPortlet)] = newPortlet
                currentObject['after']['right'] = rightAssignable.items()
        if (is_iterable_left and left_slots) or (is_iterable_right and right_slots):
            self.migrated.append(currentObject)

        if is_iterable_left:
            obj.left_slots = []

        if is_iterable_right:
            obj.right_slots = []

        transaction.commit()

    def remove_duplicated_portlets(self, obj):
        left = getUtility(IPortletManager, name='plone.leftcolumn')
        right = getUtility(IPortletManager, name='plone.rightcolumn')
        assignedPortlets = {}
        portletsToRemove = []
        try:
            rightAssignable = getMultiAdapter((obj, right), IPortletAssignmentMapping).__of__(obj)
        except:
            return
            
        for key, item in rightAssignable.items():
            shouldAssign = False
            if item.title == 'portlet_cache' or item.title == 'portlet_themes' or item.title == 'portlet_guestbook':
                shouldAssign = False
            else:
                if item.title not in assignedPortlets.keys():
                    shouldAssign = True
                    
            if shouldAssign:
                assignedPortlets[item.title] = key
            else:
                portletsToRemove.append(key)

        
        for portletToRemove in portletsToRemove:
            del rightAssignable[portletToRemove]

        if len(rightAssignable.values()) > 0:
            context = aq_inner(obj)
            inheritedPortlets = []
            while not IPloneSiteRoot.providedBy(context):
                if IAcquirer.providedBy(context):
                    context = aq_parent(aq_inner(context))
                else:
                    context = context.__parent__
                rightAssignableOfContext = getMultiAdapter((context, right), IPortletAssignmentMapping).__of__(context)
                for portlet in rightAssignableOfContext.values():
                    inheritedPortlets.append(portlet.title)
#            import pdb; pdb.set_trace()
            
            for portletToRemove in list(set(assignedPortlets.keys()).intersection(set(inheritedPortlets))):
                del rightAssignable[assignedPortlets[portletToRemove]]
#            import pdb; pdb.set_trace()
                    
    def convert_all_legacy_portlets(self):
        """Convert legacy portlets (left_slots, right_slots) in the given
        context to new-style portlets.
        Move all portlets from left to right
        """

        self.migrated = []
        self.moved = []
        self.errors = []

        cat = getToolByName(self.context,'portal_catalog')
        
#        brains = cat(portal_type='Folder')
        brains = cat()

        self.convert_legacy_portlets_for_object(self.context)
        for brain in brains:
            obj = brain.getObject()
            self.convert_legacy_portlets_for_object(obj)


        for obj in self.toCheck:
            self.remove_duplicated_portlets(obj)
            
        return self.template()



    def migrate_legacy_portlets(self):
        return """
            <html>
                <body>
                    <form method='post' action='"""+self.context.absolute_url()+"""/@@convert-all-legacy-portlets'>
                        <input type='submit' value='Migrate All Legacy Portlets'/>
                    </form>
                <body>
            </html>"""
            
