from Products.Five import BrowserView
import lxml.html
from Products.CMFCore.utils import getToolByName
from eea.themecentre.themecentre import getTheme

class ThemeMigrations(BrowserView):
    """ Themes related migrations
    """
    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.catalog = getToolByName(context, 'portal_catalog')

    def migrateThemeFolders(self):
        """ Removes the intro page as the default_page and sets
            the layout of theme folders to themecentre_view and
            sets as description for the folder the description
            of the intro page if exists, otherwise it constucts
            one from the first paragraph
        """
        query = { 'object_provides' :
                'eea.themecentre.interfaces.IThemeCentre',
                'review_state': 'published', 'path' :
                {'query': '/www/SITE/themes', 'depth': 1}}
        res = self.catalog.searchResults(query)
        res = [obj.getObject() for obj in res]
        for obj in res:
            translations = obj.getTranslations()
            values = translations.values()
            for trans in values:
                item = trans[0]
                if hasattr(item, 'intro'):
                    intro = item.intro
                else:
                    if getTheme(item) == 'technology':
                        intro = item.restrictedTraverse('environmental-technology-introduction')
                        parent = intro.aq_parent
                        parent.manage_renameObject(intro.getId(), 'intro')
                        intro = parent.intro
                    elif getTheme(item) == 'water':
                        intro = item.restrictedTraverse('welcome-to-water-information-system-for-europe-wise')
                        parent = intro.aq_parent
                        parent.manage_renameObject(intro.getId(), 'intro')
                        intro = parent.intro
                    elif getTheme(item) == 'landuse':
                        intro = item.restrictedTraverse('land-use-introduction')
                        parent = intro.aq_parent
                        parent.manage_renameObject(intro.getId(), 'intro')
                        intro = parent.intro
                    else:
                        intro = item.restrictedTraverse(item.getProperty('default_page'))

                if item.hasProperty('layout'):
                    item.manage_changeProperties({'layout' : 'themecentre_view'})
                else:
                    item.manage_addProperty('layout', 'themecentre_view', 'string')
                if item.hasProperty('default_page'):
                    item.manage_delProperties(['default_page'])
                if intro.hasProperty('layout'):
                    intro.manage_delProperties(['layout'])
                intro_desc = intro.Description()
                if intro_desc:
                    item.setDescription(intro_desc)
                else:
                    body = lxml.html.fromstring(intro.CookedBody())
                    if len(body.findall('p')) > 0:
                        first_paragraph = body.findall('p')[0].text
                    else:
                        first_paragraph = body.text
                    item.setDescription(first_paragraph)
        return "done"

    def migrateEnvironmentalThemes(self):
        """ Adds themes_view layout to themes folder and its translations
        """
        obj = self.context.restrictedTraverse('www/SITE/themes')
        translations = obj.getTranslations()
        values = translations.values()
        for trans in values:
            try:
        	item = trans[0]
        	if item.hasProperty('default_page'):
            	    item.manage_delProperties(['default_page'])
        	if item.hasProperty('layout'):
            	    item.manage_changeProperties({'layout' : 'themes_view'})
        	else:
            	    item.manage_addProperty('layout', 'themes_view', 'string')
            except:
                pass
                
        return "done"

