""" Setup handlers
"""
from Products.CMFCore.utils import getToolByName
import logging
import transaction
from eea.plone4migration.setuphandlers_data import topic_states
#from Products.CMFEditions.setuphandlers import DEFAULT_POLICIES

logger = logging.getLogger('eea.plone4migration: setuphandlers')

def migrateTopicStates(context):
    if context.readDataFile('eeaplone4migration.txt') is None:
        return

    logger.info('Updating %s Topics' % len(topic_states.keys()))

    site = context.getSite()
    cattool = getToolByName(site, 'portal_catalog')
    wftool = getToolByName(site, 'portal_workflow')

    query = {'portal_type': ['Topic'],
             'Language': 'all',}
    brains = cattool(**query)

    i = 0
    err = 0
    for k in brains:
        i += 1
        try:
            if not topic_states.has_key(k.getPath()):
                logger.info('Error, no Topic found on production with path %s' % k.getPath())
                continue

            curent_state = k.review_state
            original_state = topic_states[k.getPath()]

            if curent_state != original_state:
                obj = k.getObject()
                workflow = wftool.getWorkflowsFor(obj)[0]

                if original_state == 'published':
                    workflow.doActionFor(obj, 'publish')
                elif original_state == 'draft':
                    if curent_state == 'visible':
                        workflow.doActionFor(obj, 'submitCopyEdit')
                        workflow.doActionFor(obj, 'reject')
                    else:
                        workflow.doActionFor(obj, 'reject')
                elif original_state == 'new':
                    workflow.doActionFor(obj, 'publish')
                    workflow.doActionFor(obj, 'retract')
                    workflow.doActionFor(obj, 'enable')
                elif original_state == 'private':
                    if curent_state != 'draft':
                        workflow.doActionFor(obj, 'publishInternally')
                    workflow.doActionFor(obj, 'hide')
                elif original_state == 'published_internally':
                    if curent_state == 'draft':
                        workflow.doActionFor(obj, 'publish')
                    workflow.doActionFor(obj, 'publishInternally')
                elif original_state == 'visible':
                    workflow.doActionFor(obj, 'showPublicDraft')

                obj.reindexObject()
                logger.info('Transition made from %s to %s for %s' % (curent_state, original_state, k.getPath()))
            else:
                logger.info('No action, state equal to production for %s' % k.getPath())
        except Exception, workflow_err:
            logger.info('Transition failed from %s to %s for %s' % (curent_state, original_state, k.getPath()))
            logger.info('ERROR: %s' % k.getPath())
            logger.exception(workflow_err)
            err += 1

        if i == 10:
            transaction.commit()
            i = 0

    all_b = len(brains)
    logger.info('Topic states updated. Total %s. Errors %s' % (all_b, err))


def migrate_soerCountries(context):
    if context.readDataFile('eeaplone4migration.txt') is None:
        return

    logger.info('Updating soer countries default view')

    site = context.getSite()
    cattool = getToolByName(site, 'portal_catalog')

    query = {'portal_type': ['SOERCountry'],
             'Language': 'all',}
    brains = cattool(**query)

    for k in brains:
        obj = k.getObject()
        if obj.hasProperty('layout') and \
                obj.getProperty('layout') != 'soercountry_view':
            obj._setPropValue('layout', 'soercountry_view')
            logger.info(u"Changed layout to soercountry_view for %s" % obj)

#def setupVersioningPolicy(context):
#    """ Setup versioning policy
#    """
#    if context.readDataFile('eeaplone4migration.txt') is None:
#        return
#
#    logger.info('Setting versioning policy.')
#
#    site = context.getSite()
#    portal_repository = getToolByName(site, 'portal_repository')
#    EEA_CUSTOM_CONTENT_TYPES = ['FlashFile', 'Article', 'AssessmentPart',
#        'CallForInteres', 'CallForTender', 'CommonalityReport', 'DiversityReport',
#        'Data', 'DataFile', 'DataTable', 'EEAVacancy', 'EEAFigure', 'EEAFigureFile',
#        'EcoTip', 'ExternalDataSpec', 'EyewitnessStory', 'FactSheetDocument',
#        'File', 'GIS Application', 'Highlight', 'Image', 'Assessment',
#        'IndicatorFactSheet', 'KeyMessage', 'RelatedIndicatorLink', 'MethodologyReference',
#        'FlexibilityReport', 'PolicyDocumentReference', 'PolicyQuestion',
#        'PressRelease', 'Promotion', 'QuickEvent', 'RationaleReference',
#        'SOERCountry', 'SOERKeyFact', 'SOERMessage', 'Speech', 'WorkItem' ]
#
#    versionable_types = list(portal_repository.getVersionableContentTypes())
#    versionable_types.extend(EEA_CUSTOM_CONTENT_TYPES)
#    portal_repository.setVersionableContentTypes(versionable_types)
#
#    for ctype in EEA_CUSTOM_CONTENT_TYPES:
#        for policy_id in DEFAULT_POLICIES:
#            portal_repository.addPolicyForContentType(ctype, policy_id)
#        logger.info('Versioning activated for %s' % ctype)
#
#    logger.info('Done setting versioning policy.')
